@@ -198,22 +198,25 @@ static void speller_dealloc(PyObject* self) {
 }
 
 /* ConfigKeys *****************************************************************/
-static PyObject* configkeys(PyObject* _) {
-
+static PyObject* configkeys_helper(PyObject* self) {
 	AspellConfig* config;
 	AspellKeyInfoEnumeration *keys_enumeration;
 	AspellStringList* lst;
 	AspellMutableContainer* amc;
 	const AspellKeyInfo *key_info;
 
-	PyObject *key_list, *obj;
+	PyObject *dict = 0, *obj = 0, *value = 0;
 	const char*  string;
 	unsigned int integer;
 	unsigned int boolean;
 
 	char *key_type = 0;
 
-	config = new_aspell_config();
+	if (self)
+		config = aspell_speller_config(Speller(self));
+	else
+		config = new_aspell_config();
+
 	if (config == NULL) {
 		PyErr_SetString(_AspellModuleException, "can't create config");
 		return NULL;
@@ -221,11 +224,17 @@ static PyObject* configkeys(PyObject* _) {
 
 	keys_enumeration = aspell_config_possible_elements(config, 1);
 	if (!keys_enumeration) {
+		if (!self) delete_aspell_config(config);
 		PyErr_SetString(_AspellConfigException, "can't get list of config keys");
 		return NULL;
 	}
 
-	key_list = PyList_New(0);
+	dict = PyDict_New();
+	if (dict == NULL) {
+		if (!self) delete_aspell_config(config);
+		return NULL;
+	}
+		
 	while ((key_info = aspell_key_info_enumeration_next(keys_enumeration))) {
 
 		/* key type -> string */
@@ -234,13 +243,13 @@ static PyObject* configkeys(PyObject* _) {
 				key_type = "string";
 				string   = aspell_config_retrieve(config, key_info->name);
 				if (aspell_config_error(config) != NULL) goto config_get_error;
-				obj      = PyString_FromString( string );
+				obj      = PyUnicode_FromString( string );
 				break;
 			case AspellKeyInfoInt:
 				key_type = "integer";
 				integer  = aspell_config_retrieve_int(config, key_info->name);
 				if (aspell_config_error(config) != NULL) goto config_get_error;
-				obj      = PyInt_FromLong( integer );
+				obj      = PyLong_FromLong( integer );
 				break;
 			case AspellKeyInfoBool:
 				key_type = "boolean";
@@ -260,103 +269,120 @@ static PyObject* configkeys(PyObject* _) {
 				break;
 		}
 
-		if (PyList_Append(key_list, Py_BuildValue("(ssOs)", key_info->name, key_type, obj, key_info->desc ? key_info->desc : "internal")) == -1) {
-			PyErr_SetString(PyExc_Exception, "It is almost impossible, but happend! Can't append element to the list.");
-			delete_aspell_key_info_enumeration(keys_enumeration);
-			Py_DECREF(key_list);
-			return NULL;
+		/* value */
+		value = Py_BuildValue("(sOs)",
+							key_type,
+							obj,
+							key_info->desc ? key_info->desc : "internal"
+		);
+
+		if (value) {
+			if (PyDict_SetItemString(dict, key_info->name, value)) {
+				goto python_error;
+			}
+			else
+				Py_DECREF(value);
 		}
+		else
+			goto python_error;
 	}
+	
 	delete_aspell_key_info_enumeration(keys_enumeration);
-	delete_aspell_config(config);
-	return key_list;
+	if (!self) delete_aspell_config(config);
+	return dict;
 
 config_get_error:
 	PyErr_SetString(_AspellConfigException, aspell_config_error_message(config));
+python_error:
 	delete_aspell_key_info_enumeration(keys_enumeration);
-	delete_aspell_config(config);
-	Py_DECREF(key_list);
+	if (!self) delete_aspell_config(config);
+	Py_DECREF(dict);
 	return NULL;
 }
 
+
+static PyObject* configkeys(PyObject* _) {
+	return configkeys_helper(NULL);
+}
+
 /* method:ConfigKeys **********************************************************/
 static PyObject* m_configkeys(PyObject* self, PyObject* args) {
+	return configkeys_helper(self);
+}
 
+/* method:setConfigKey ********************************************************/
+static PyObject* m_set_config_key(PyObject* self, PyObject* args) {
 	AspellConfig* config;
-	AspellKeyInfoEnumeration *keys_enumeration;
-	AspellStringList* lst;
-	AspellMutableContainer* amc;
-	const AspellKeyInfo *key_info;
+	const AspellKeyInfo* info;
+	char* key;
+	char* string;
+	long  number;
+	char  buffer[32];
 
-	PyObject *key_list, *obj;
-	const char*  string;
-	unsigned int integer;
-	unsigned int boolean;
+	PyObject* arg1;
+
+	if (PyTuple_Size(args) != 2) {
+		PyErr_Format(PyExc_TypeError, "expected two arguments");
+		return NULL;
+	}
+
+	if (!PyArg_ParseTuple(args, "sO", &key, &arg1)) {
+		PyErr_Format(PyExc_TypeError, "first argument must be a string");
+		return NULL;
+	}
 
-	char *key_type = 0;
 
 	config = aspell_speller_config(Speller(self));
-	if (config == NULL) {
-		PyErr_SetString(_AspellModuleException, "can't create config");
+	info   = aspell_config_keyinfo(config, key);
+	if (aspell_config_error(config) != 0) {
+		PyErr_SetString(_AspellConfigException, aspell_config_error_message(config));
 		return NULL;
 	}
 
-	keys_enumeration = aspell_config_possible_elements(config, 1);
-	if (!keys_enumeration) {
-		PyErr_SetString(_AspellConfigException, "can't get list of config keys");
-		return NULL;
-	}
+	switch (info->type) {
+		case AspellKeyInfoList:
+			// Can't figure out the splitting char for lists,
+			// it seems to be the ':', but doesn't work.
+		case AspellKeyInfoString:
+			string = PyString_AsString(arg1);
+			if (string == NULL) {
+				PyErr_Format(PyExc_TypeError, "second argument have to be string");
+				return NULL;
+			}
 
-	key_list = PyList_New(0);
-	while ((key_info = aspell_key_info_enumeration_next(keys_enumeration))) {
+			aspell_config_replace(config, key, string);
+			break;
 
-		/* key type -> string */
-		switch (key_info->type) {
-			case AspellKeyInfoString:
-				key_type = "string";
-				string   = aspell_config_retrieve(config, key_info->name);
-				if (aspell_config_error(config) != NULL) goto config_get_error;
-				obj      = PyString_FromString( string );
-				break;
-			case AspellKeyInfoInt:
-				key_type = "integer";
-				integer  = aspell_config_retrieve_int(config, key_info->name);
-				if (aspell_config_error(config) != NULL) goto config_get_error;
-				obj      = PyInt_FromLong( integer );
-				break;
-			case AspellKeyInfoBool:
-				key_type = "boolean";
-				boolean  = aspell_config_retrieve_bool(config, key_info->name);
-				if (aspell_config_error(config) != NULL) goto config_get_error;
-				obj      = PyBool_FromLong( boolean );
-				break;
-			case AspellKeyInfoList:
-				key_type = "list";
-				lst = new_aspell_string_list();
-				amc = aspell_string_list_to_mutable_container(lst);
-				aspell_config_retrieve_list(config, key_info->name, amc);
-				if (aspell_config_error(config) != NULL) goto config_get_error;
+		case AspellKeyInfoInt:
+			number = PyLong_AsLong(arg1);
+			if (number == -1 && PyErr_Occurred()) {
+				return NULL;
+			}
 
-				obj = AspellStringList2PythonList(lst);
-				delete_aspell_string_list(lst);
-				break;
-		}
+			snprintf(buffer, 32, "%ld", number);
+			aspell_config_replace(config, key, buffer);
+			break;
 
-		if (PyList_Append(key_list, Py_BuildValue("(ssO)", key_info->name, key_type, obj)) == -1) {
-			PyErr_SetString(PyExc_Exception, "It is almost impossible, but happend! Can't append element to the list.");
-			delete_aspell_key_info_enumeration(keys_enumeration);
-			Py_DECREF(key_list);
+		case AspellKeyInfoBool:
+			if (PyBool_Check(arg1)) {
+				aspell_config_replace(config, key, (arg1 == Py_True) ? "true" : "false");
+			} else {
+				PyErr_Format(PyExc_TypeError, "second argument have to be boolean");
+				return NULL;
+			}
+			break;
+
+		default:
+			PyErr_Format(_AspellModuleException, "unsupported aspell config item type");
 			return NULL;
-		}
 	}
-	delete_aspell_key_info_enumeration(keys_enumeration);
-	return key_list;
 
-config_get_error:
-	PyErr_SetString(_AspellConfigException, aspell_config_error_message(config));
-	delete_aspell_key_info_enumeration(keys_enumeration);
-	Py_DECREF(key_list);
-	return NULL;
+	if (aspell_config_error(config) != 0) {
+		PyErr_SetString(_AspellConfigException, aspell_config_error_message(config));
+		return NULL;
+	}
+
+	Py_RETURN_NONE;
 }
 
 /* method:check ***************************************************************/
@@ -490,11 +516,18 @@ static PyMethodDef aspell_object_methods[] = {
 		"ConfigKeys",
 		(PyCFunction)m_configkeys,
 		METH_VARARGS,
-		"ConfigKeys() => list of config keys\n"
-		"The list's item is a 3-touple:\n"
-		"\t1. key name\n"
-		"\t2. key type={string|integer|boolean|list}\n"
-		"\t4. current value"
+		"ConfigKeys() => dictionary of config keys\n"
+        "Keys are string, values are 3-touple:\n"
+		"\t1. key type={string|integer|boolean|list}\n"
+		"\t2. current value\n"
+		"\t3. description (if 'internal' no description available)"
+	},
+	{
+		"setConfigKey",
+		(PyCFunction)m_set_config_key,
+		METH_VARARGS,
+		"changeConfig(key, value)\n"
+		"Sets a new config value for given key"
 	},
 	{
 		"check",
@@ -600,12 +633,11 @@ static PyMethodDef aspell_methods[] = {
 		"ConfigKeys",
 		(PyCFunction)configkeys,
 		METH_VARARGS,
-		"ConfigKeys() => list of config keys\n"
-		"The list's item is a 4-touple:\n"
-		"\t1. key name\n"
-		"\t2. key type={string|integer|boolean|list}\n"
-		"\t3. default value\n"
-		"\t4. short description ('internal' for undocumented options)."
+		"ConfigKeys() => dictionary of config keys\n"
+        "Keys are string, values are 3-touple:\n"
+		"\t1. key type={string|integer|boolean|list}\n"
+		"\t2. current value\n"
+		"\t3. description (if 'internal' no description available)"
 	},
 	{NULL, NULL, 0, NULL}
 };
